unit TestMain;
 
{$mode objfpc}{$H+}
 
interface
 
uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, fgl;
 
type
 
  { TSpriteTestForm }
 
  TSprite = class
  protected
    FBitmap:TBitmap;
  public
    Rect:TRect;
    Highlighted:Boolean;
    constructor Create(ABitmap:TBitmap;AIndex:Integer;ARect:TRect);
    destructor Destroy;override;
    procedure Draw(ACanvas:TCanvas);
    function HitTest(APoint:TPoint):Boolean;
    property Bitmap:TBitmap read FBitmap;
  end;
 
  TSpriteList = specialize TFPGList<TSprite>;
 
  TSpriteTestForm = class(TForm)
    SpriteImage: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpriteImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SpriteImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure SpriteImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { private declarations }
  public
    { public declarations }
    Drag:Boolean;
    DragSprite:TSprite;
    DragStart:TPoint;
    SpriteStart:TRect;
    SpriteBitmap:TBitmap;
    SpriteList:TSpriteList;
    procedure UpdateRect;
  end;
 
var
  SpriteTestForm: TSpriteTestForm;
 
implementation
 
{$R *.lfm}
 
{ TSpriteTestForm }
 
 
procedure TSpriteTestForm.FormCreate(Sender: TObject);
  var Bitmap:TBitmap;
begin
  Randomize;
  Bitmap := TBitmap.Create;
  Bitmap.Width := 640;
  Bitmap.Height := 480;
  Bitmap.Canvas.Brush.Color := clGreen;
  Bitmap.Canvas.FillRect(0, 0, 640, 480);
  SpriteImage.Picture.Bitmap := Bitmap;
  SpriteBitmap := TBitmap.Create;
  SpriteBitmap.LoadFromFile('MS.bmp');
  SpriteList := TSpriteList.Create;
end;
 
procedure TSpriteTestForm.FormDestroy(Sender: TObject);
  var I:Integer;
begin
  SpriteBitmap.Free;
  for I := 0 to SpriteList.Count - 1 do begin
    SpriteList[I].Free;
  end;
  SpriteList.Free;
end;
 
procedure TSpriteTestForm.SpriteImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  var Sprite:TSprite;I:Integer;
begin
  if Button = mbRight then begin
    Sprite := TSprite.Create(SpriteBitmap, Random((12 * 8) - 1), TRect.Create(TPoint.Create(X - 40, Y - 40), 81, 81));
    SpriteList.Insert(0, Sprite);
    SpriteImageMouseMove(Sender, Shift, X, Y);
  end;
  DragSprite := nil;
  Drag := False;
  if Button = mbLeft then begin
    for I := 0 to SpriteList.Count - 1 do begin
      if SpriteList[I].HitTest(TPoint.Create(X, Y)) then begin
        DragSprite := SpriteList[I];
        DragStart := TPoint.Create(X, Y);
        SpriteStart := SpriteList[I].Rect;
        Break;
      end;
    end;
  end;
end;
 
procedure TSpriteTestForm.SpriteImageMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
  var I:Integer;Found:Boolean;
  Rect:TRect;Dist:TPoint;
begin
  Rect := TRect.Create(0, 0, 0, 0);
  if Assigned(DragSprite) then begin
    Dist := TPoint.Create(X, Y) - DragStart;
    if not Drag then begin
      if (Abs(Dist.X) > 5) or (Abs(Dist.Y) > 5) then begin
        Drag := True;
      end;
    end
    else begin
      Rect.Union(DragSprite.Rect);
      DragSprite.Rect := SpriteStart;
      DragSprite.Rect.Offset(Dist);
      Rect.Union(DragSprite.Rect);
    end;
  end;
  Found := False;
  for I := 0 to SpriteList.Count - 1 do begin
    if Found then begin
      if SpriteList[I].Highlighted then begin
        SpriteList[I].Highlighted := False;
        Rect.Union(SpriteList[I].Rect);
      end;
    end
    else begin
      if SpriteList[I].HitTest(TPoint.Create(X, Y)) then begin
        if not SpriteList[I].Highlighted then begin
          SpriteList[I].Highlighted := True;
          Rect.Union(SpriteList[I].Rect);
        end;
        Found := True;
      end
      else begin
        if SpriteList[I].Highlighted then begin
          SpriteList[I].Highlighted := False;
          Rect.Union(SpriteList[I].Rect);
        end;
      end;
    end;
  end;
  UpdateRect;
end;
 
procedure TSpriteTestForm.SpriteImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  var I:Integer;
begin
  if Button = mbLeft then begin
    if not Drag then begin
      for I := 0 to SpriteList.Count - 1 do begin
        if SpriteList[I].HitTest(TPoint.Create(X, Y)) then begin
          SpriteList.Move(I, 0);
        end;
      end;
      SpriteImageMouseMove(Sender, Shift, X, Y);
    end;
    DragSprite := nil;
  end;
end;
 
procedure TSpriteTestForm.UpdateRect;
  var I:Integer;
begin
  with SpriteImage.Picture.Bitmap.Canvas do begin
    Brush.Color := clGreen;
    FillRect(ClipRect);
  end;
  for I := SpriteList.Count - 1 downto 0 do begin
    SpriteList[I].Draw(SpriteImage.Picture.Bitmap.Canvas);
  end;
end;
 
constructor TSprite.Create(ABitmap:TBitmap;AIndex:Integer;ARect:TRect);
  var Dest, Src:TRect;
begin
  Rect := ARect;
  FBitmap := TBitmap.Create;
  FBitmap.Width := 81;
  FBitmap.Height := 81;
  Src := TRect.Create(TPoint.Create(0, 0), 81, 81);
  Dest := Src;
  Src.Offset(5 + 81 * (AIndex mod 12), 5 + 81 * (AIndex div 12));
  FBitmap.Canvas.CopyRect(Dest, ABitmap.Canvas, Src);
  FBitmap.Mask(clBlack);
end;
 
destructor TSprite.Destroy;
begin
  Bitmap.Free;
  inherited Destroy;
end;
 
procedure TSprite.Draw(ACanvas:TCanvas);
begin
  ACanvas.Draw(Rect.Left, Rect.Top, Bitmap);
  if Highlighted then begin
    ACanvas.DrawFocusRect(Rect);
  end;
end;
 
function TSprite.HitTest(APoint:TPoint):Boolean;
begin
  Result := False;
  if Rect.Contains(APoint) then begin
    APoint := APoint - Rect.TopLeft;
    Result := Bitmap.Canvas.Pixels[APoint.X, APoint.Y] <> clBlack;
  end;
end;
 
end.
